; -- wxWidgets Package Management suite InnoSetup --
; Author: Francesco Montorsi
; Date: 20/9/2006
; $Id: wxp.iss,v 1.4 2007/02/02 09:41:58 frm Exp $

; this should be the value of the VERSION_NUMBER symbol defined in cmnsymbols.h
#define VERSION          "0.2"

; this should be the value of the APPNAME symbol defined in cmnsymbols.h
#define DIRNAME          "wxPackageManagementSuite"

; the name of the suite installed
#define NAME             "wxWidgets package management suite"

#define COPYRIGHT        "Copyright (C) 2006-2007 wxWPM developers."

[Setup]

; version-related
VersionInfoVersion={#VERSION}
AppVerName={#NAME} {#VERSION}
OutputBaseFilename=wxwpm-{#VERSION}-setup

; compiler-related
SourceDir=..\..
OutputDir=..\..\..
Compression=lzma/fast
SolidCompression=yes

; installer-related
AppName={#NAME}
ChangesAssociations=yes
DefaultDirName={pf}\{#DIRNAME}
DefaultGroupName=wxWidgets package management
AllowNoIcons=yes
LicenseFile=docs\licence.txt
ShowLanguageDialog=yes

; cosmetic
WindowShowCaption=true
WindowStartMaximized=false
FlatComponentsList=true
WindowResizable=false
ShowTasksTreeLines=false
SetupIconFile=src\pkg\bitmaps\wxp.ico

VersionInfoCopyright={#COPYRIGHT}
AppCopyright={#COPYRIGHT}



[Components]
Name: main; Description: Main Files; Types: full compact custom; Flags: fixed
Name: help; Description: Help Files; Types: full compact
Name: samples; Description: Sample Files; Types: full


[Files]

; main component
Source: *.exe; DestDir: {app}; Components: main
Source: xml\*; DestDir: {app}\xml; Components: main
Source: images\*.gif; DestDir: {app}\images; Components: main
Source: images\*.png; DestDir: {app}\images; Components: main
Source: defaultpages\*; DestDir: {app}\defaultpages; Components: main
Source: viewformats\*; DestDir: {app}\viewformats; Flags: recursesubdirs; Components: main

; for the uninstall link
Source: src\pkg\bitmaps\win-uninstall.ico; DestDir: {app}; Components: main

; help component
Source: docs\images\*; DestDir: {app}\docs\images; Flags: recursesubdirs; Components: help
Source: docs\user\*; DestDir: {app}\docs\user; Flags: recursesubdirs; Components: help
Source: docs\api\*; DestDir: {app}\docs\api; Flags: recursesubdirs; Components: help
Source: Readme.txt; DestDir: {app}; Components: help

; sample component
Source: samples\*; DestDir: {app}\samples; Flags: recursesubdirs; Components: samples


[Messages]
BeveledLabel=wxWidgets package management


[Tasks]
Name: startmenu; Description: Create icons in Start menu; GroupDescription: Additional icons:; Components: main
Name: desktopicon; Description: Create a &desktop icon; GroupDescription: Additional icons:; Components: main
Name: quicklaunchicon; Description: Create a &Quick Launch icon; GroupDescription: Additional icons:; Components: main; Flags: unchecked
Name: associateopen; Description: Associate OPEN action for .wxp and .wxz files; GroupDescription: Other tasks:; Components: main
Name: associateedit; Description: Associate EDIT action for .wxp and .wxz files; GroupDescription: Other tasks:; Components: main
Name: updatepath; Description: Add WXP command-line program's folder to the PATH; GroupDescription: Other tasks:; Components: main


[Icons]
Name: {group}\Packager; Filename: {app}\packager.exe; WorkingDir: {app}; Components: main; Tasks: startmenu
Name: {group}\Package Manager; Filename: {app}\packagemanager.exe; WorkingDir: {app}; Components: main; Tasks: startmenu
Name: {userdesktop}\wxPackager; Filename: {app}\packager.exe; WorkingDir: {app}; Components: main; Tasks: desktopicon
Name: {userdesktop}\wxPackageManager; Filename: {app}\packagemanager.exe; WorkingDir: {app}; Components: main; Tasks: desktopicon
Name: {userappdata}\Microsoft\Internet Explorer\Quick Launch\wxPackager; Filename: {app}\packager.exe; Tasks: quicklaunchicon
Name: {userappdata}\Microsoft\Internet Explorer\Quick Launch\wxPackageManager; Filename: {app}\packagemanager.exe; Tasks: quicklaunchicon

Name: {group}\Readme.txt; Filename: {app}\Readme.txt; Components: help; Tasks: startmenu
Name: {group}\User guide; Filename: {app}\docs\user\index.html; Components: help; Tasks: startmenu
Name: {group}\Uninstall; Filename: {uninstallexe}; IconFilename: {app}\win-uninstall.ico


[Registry]

; .WXP extension
; ==============

; create .wxp filetype and associate an icon to it
Root: HKCR; Subkey: .wxp; ValueType: string; ValueName: ; ValueData: wxPackage; Flags: uninsdeletevalue; Tasks: associateedit or associateopen
Root: HKCR; Subkey: wxPackage; ValueType: string; ValueName: ; ValueData: wxWidgets package descriptor; Flags: uninsdeletekey; Tasks: associateedit or associateopen
Root: HKCR; Subkey: wxPackage\DefaultIcon; ValueType: string; ValueName: ; ValueData: {app}\packager.exe,0; Tasks: associateedit or associateopen

; associate the EDIT action for .wxp extensions with the wxPackager
Root: HKCR; Subkey: wxPackage\shell\edit\command; ValueType: string; ValueName: ; ValueData: """{app}\packager.exe"" ""%1"""; Tasks: associateedit

; associate the OPEN action for .wxp extensions with the wxPackager
Root: HKCR; Subkey: wxPackage\shell\open\command; ValueType: string; ValueName: ; ValueData: """{app}\packager.exe"" ""%1"""; Tasks: associateopen



; .WXZ extension
; ==============

; create .wxz filetype and associate an icon to it
Root: HKCR; Subkey: .wxz; ValueType: string; ValueName: ; ValueData: wxCompressedPackage; Flags: uninsdeletevalue; Tasks: associateedit or associateopen
Root: HKCR; Subkey: wxCompressedPackage; ValueType: string; ValueName: ; ValueData: wxWidgets compressed package; Flags: uninsdeletekey; Tasks: associateedit or associateopen
Root: HKCR; Subkey: wxCompressedPackage\DefaultIcon; ValueType: string; ValueName: ; ValueData: {app}\packagemanager.exe,0; Tasks: associateedit or associateopen

; associate the EDIT action for .wxz extensions with the wxPackageManager
Root: HKCR; Subkey: wxCompressedPackage\shell\edit\command; ValueType: string; ValueName: ; ValueData: """{app}\packagemanager.exe"" ""%1"""; Tasks: associateedit

; associate the OPEN action for .wxz extensions with the wxPackageManager
Root: HKCR; Subkey: wxCompressedPackage\shell\open\command; ValueType: string; ValueName: ; ValueData: """{app}\packagemanager.exe"" ""%1"""; Tasks: associateopen





; UPDATE PATH stuff
; =================


[Code]

#include "modifypath.iss"

procedure CurStepChanged(CurStep: TSetupStep);
begin
  case CurStep of
    ssPostInstall:
      begin

        if IsTaskSelected('updatepath') then
          // if logged in as the admin we can add the path to all users
          if IsAdminLoggedOn then
            ModifyPath('{app}', pmAddToEnd, psAllUsers)
          else
            ModifyPath('{app}', pmAddToEnd, psCurrentUser)
            
      end
  end
end;

procedure CurUninstallStepChanged(CurUninstallStep: TUninstallStep);
begin
  case CurUninstallStep of
    usPostUninstall:
      begin
        if IsAdminLoggedOn then
          ModifyPath('{app}', pmRemove, psAllUsers)
        else
          ModifyPath('{app}', pmRemove, psCurrentUser)
      end
  end
end;



